% Description of this demo:
%  (1) In this demo we compare the performance of tMFOCUSS and MFOCUSS. 
%  (2) To avoid the disturbance of the incorrect selection of regularization
%      parameters (i.e. lambda) for both algorithms, we select 5 candidate 
%      values for lambda for each algorithm. 
%  (3) For each candidate value of lambda, we run each algorithm for 20
%      trials.
%  (4) Finally, we plot the averaged results in terms of failure rates and
%      MSE.
%
% ====== The whole experiment takes about 1 minute. ======
%
% Author: Zhilin Zhang (z4zhang@ucsd.edu)
% Date  : 05/12/2011
%

clear; clc; close all;


% problem dimension
N = 50;   % row number of the dictionary matrix
M = 200;  % column number of the dictionary matrix
K = 20;   % number of nonzero rows in the solution matrix
L = 4;    % number of measurement vectors

% candidate values for the regularization parameter (lambda)
LAMBDA = logspace(-6,-3, 5);

% We repeat the experiment for 20 trials to see the statistical result
for it = 1:15
    fprintf('\n\nRunning %d:\n',it);

    % dictionary matrix with columns draw uniformly from the surface of a unit hypersphere
    Phi = randn(N,M);
    Phi = Phi./(ones(N,1)*sqrt(sum(Phi.^2)));

    % Generate K sources. Each source is AR(1) source with the AR
    % coefficient being 0.9
    beta = ones(K,1)*0.9;
    nonzeroW(:,1) = randn(K,1);
    alpha = ones(K,1);
    for i = 2 : L*50
        nonzeroW(:,i) = beta .* nonzeroW(:,i-1) + sqrt(1-beta.^2).*(sqrt(alpha).*randn(K,1));
    end
    nonzeroW = nonzeroW(:,end-L+1:end);   % get stable AR sources
    nonzeroW = nonzeroW./( sqrt(sum(nonzeroW.^2,2)) * ones(1,L) );   % normalize along row

    % select active sources at random locations
    ind = randperm(M);
    indice = ind(1:K);
    Wgen = zeros(M,L);
    Wgen(indice,:) = nonzeroW;

    % noiseless signal
    signal = Phi * Wgen;

    % noise
    stdvariance = 0.01;
    noise = randn(N,L) * stdvariance;

    % noisy signal
    Y = signal + noise;


    for n = 1 : length(LAMBDA)
        lambda = LAMBDA(n);
        %============================ tMFOCUSS ==========================
        [X_tf, gamma_ind_tf, gamma_est_tf, count_tf] = tMFOCUSS(Phi, Y, lambda);

        % Failure Rate
        F_tf = perfSupp(X_tf,indice,'firstlargest', K);
        FR_tMFOCUSS(it,n) = (F_tf~=1);

        MSE_tMFOCUSS(it,n) = (norm(Wgen - X_tf,'fro')/norm(Wgen,'fro'))^2;

        %     fprintf(' tMFOCUSS: iter = %3d; Findex = %3.2f, Ave MSE = %6.5f; Ave Failure-Rate = %6.5f; Ave Time = %4.3f\n',...
        %         count_tf,F_tf,mean(MSE_tMFOCUSS),mean(FR_tMFOCUSS),mean(TIME_tf));

    end
    fprintf('Regular Path of tMFOCUSS (in terms of MSE) \n');
    mean(MSE_tMFOCUSS,1)

    %============================ MFOCUSS ==========================
    for n = 1 : length(LAMBDA)
        lambda = LAMBDA(n);
        [X_focuss, gamma_ind_focuss, gamma_est_focuss, count_focuss] = MFOCUSS(Phi, Y, lambda);


        F_focuss = perfSupp(X_focuss,indice,'firstlargest', K);
        FR_MFOCUSS(it,n) = (F_focuss~=1);

        MSE_MFOCUSS(it,n) = (norm(Wgen - X_focuss,'fro')/norm(Wgen,'fro'))^2;

        %     fprintf('  MFOCUSS: iter = %3d; Findex = %3.2f, Ave MSE = %6.5f; Ave Failure-Rate = %6.5f; Ave Time = %4.3f\n',...
        %         count_focuss,F_focuss,mean(MSE_MFOCUSS),mean(FR_MFOCUSS),mean(TIME_focuss));
    end

    fprintf('Regular Path of MFOCUSS (in terms of MSE) \n');
    mean(MSE_MFOCUSS,1)


end


figure(1);
semilogx(LAMBDA,mean(FR_tMFOCUSS),'r','linewidth',3);
hold on;semilogx(LAMBDA,mean(FR_MFOCUSS),'k','linewidth',3);
legend('tMFOCUSS','MFOCUSS','Location','Northwest');
title('\bf\fontsize{14}Performance Measured by Failure Rate');
xlabel('\bf\fontsize{12}lambda');
ylabel('\bf\fontsize{12}Failure Rate'); axis tight;

figure(2);
semilogx(LAMBDA,mean(MSE_tMFOCUSS),'r','linewidth',3);
hold on;semilogx(LAMBDA,mean(MSE_MFOCUSS),'k','linewidth',3);
legend('tMFOCUSS','MFOCUSS','Location','Northwest');
title('\bf\fontsize{14}Performance Measured by Mean Square Error');
xlabel('\bf\fontsize{12}lambda');
ylabel('\bf\fontsize{12}MSE'); axis tight;

