function [performanceInSupp, recoveryRate] = perfSupp(estSource, trueSourceIndex, varargin)
% Evaluate performance of support recovery (useful in noisy case)
%
% INPUT Parameters:
%   estSource               : estimated sources
%   trueSourceIndex         : true support location index
%   varargin{1} == 'firstLargest' & varargin{2} == D 
%       : select the support of D largest sources (in terms of 2-norm)
%   varargin{1} == 'largerThan' & varargin{2} == D
%       : select the support of elements larger than D
%
% OUTPUT Parameters:
%   performanceInSupp       : performance in support recovery (=1: perfect)
%   recoveryRate            : performance in sucessfully finding the
%                             nonzero elements in source vector (or
%                             matrix). But even recoveryInSupp = 1, the
%                             algorithm may set other entries nonzero,
%                             which should be zero.
%
% Use:
%     perf = perfSupp(estSource, trueSourceIndex, 'firstLargest', 10);
%     [perfInSupp,recRate] = perfSupp(estSource, trueSourceIndex, 'largerThan', 1e-5);
% 
%   Author: Zhilin Zhang
%   Date  : July, 2010
% Version : 1.1



if length(varargin) == 2
    switch lower(varargin{1})
        case 'firstlargest'
            numThreshold = varargin{2};
            estSource = sum(estSource.^2,2);
            [sortedSource,sortInd] = sort(estSource, 1, 'descend');
            ind = sortInd(1:numThreshold);

        case 'largerthan'
            valThreshold = varargin{2};
            estSource = sum(estSource.^2,2);
            ind = find(estSource >= valThreshold);

    end
else
    error('Optional parameters are wrong!\n');
end

commonSupp = intersect(ind,trueSourceIndex);
ind1 = length(commonSupp)/length(ind);
recoveryRate = length(commonSupp)/length(trueSourceIndex);

if isempty(commonSupp), 
    performanceInSupp = 0; 
else
    performanceInSupp = 2 * ind1 * recoveryRate/(ind1 + recoveryRate);
end



